/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "backendjobsqueue.h"
#include "backend.h"

#include <kdebug.h>
#include <kjob.h>

#include "backendjobs.h"
#include "waitwidget.h"

namespace KBlogger
{

class backendJobsQueue_p
{
public:

    backendJobsQueue_p(): mStop(false), mRunning(false) {
        kDebug();
    };

    ~backendJobsQueue_p() {};

    void setStop(bool value) {
        mStop = value;
    };

    bool isStopped() {
        return mStop;
    }

    void setRunning(bool value) {
        mRunning = value;
    }

    bool isRunning() {
        return mRunning;
    }

private:
    bool mStop;
    bool mRunning;
};



backendJobsQueue::backendJobsQueue(QObject *parent): QObject(parent),
        currentJob(0), mWaitWidget(0)
{
    kDebug();

    d = new backendJobsQueue_p();
}

backendJobsQueue::~backendJobsQueue()
{
    kDebug();
    delete d;
}

bool backendJobsQueue::addJob(backendJobs* job)
{
    kDebug();

    if (d->isRunning()) {
        kError() << "This job cannot be queued while other jobs are running" << endl;
        return false;
    }
    mPendingJobs << job;
    return true;
}

void backendJobsQueue::run()
{
    kDebug();
    if (d->isRunning()) {
        kError() << "Jobs run already" << endl;
        return;
    }
    emit jobsStarted(mPendingJobs.count());
    d->setRunning(true);

    runsFirstJob();
}

bool backendJobsQueue::stop()
{
    kDebug();

    if (d->isStopped()) {
        kDebug() << "mStop is already TRUE";
        return false;
    }
    d->setStop(true);
    return true; //ok, queue stopped.
}


void backendJobsQueue::clear()
{
    kDebug();
    mPendingJobs.clear();
}

/*
bool backendJobsQueue::killCurrentJob()
{
   kDebug();
   if ( !d->isRunning() ) return false;
   d->setRunning(false);
   if ( currentJob )
     return currentJob->kill();
   else
     return false;
}
*/

bool backendJobsQueue::isRunning()
{
    kDebug();
    return d->isRunning();
}

int backendJobsQueue::queueSize()
{
    kDebug();
    return mPendingJobs.size();
}

void backendJobsQueue::runsFirstJob()
{
    kDebug();
    if ( d->isStopped() ) {
        emit jobsStopped();
        d->setRunning(false);
        d->setStop(false);
        hideWaitWidget();
        return;
    }

    if (!mPendingJobs.isEmpty()) {
        currentJob = mPendingJobs.takeFirst();
        QString message = i18np("1 job left", "%1 jobs left", mPendingJobs.count());
        backend::self()->showStatusBarMessage(message);
        Q_ASSERT(currentJob);
        showUpdatedWaitWidget( currentJob->getWaitMessage(),
                               currentJob->getWidgetParent() );
        connect(currentJob, SIGNAL( result( KJob * ) ),
                this, SLOT( slotResult( KJob * ) ) );
        currentJob->start();
    } else {
        emit jobsDone();
        hideWaitWidget();
        d->setRunning(false);
    }
}

void backendJobsQueue::slotResult( KJob * kjob)
{
    kDebug();
    currentJob->disconnect( );
    emit jobDone();
    runsFirstJob();
    //Q_UNUSED(kjob);
    backendJobs* job;
    job = qobject_cast<backendJobs*>(kjob);
    if ( kjob->error() ) {
        // An error occurred
        emit jobsDone();
        hideWaitWidget(); //TODO Remove this and use a Error Message with a OK Button.
        d->setRunning(false);

    }
}

///waitWidget functions
void backendJobsQueue::showUpdatedWaitWidget( const QString& text, QWidget* caller )
{
    kDebug()  << text << mWaitWidget;

    if ( !mWaitWidget ) {
        kDebug() << "Creo";
        mWaitWidget = waitWidget::self(caller);
    } else
        kDebug() << "Esiste";

    Q_ASSERT(mWaitWidget);

    mWaitWidget->update( text, caller );
    mWaitWidget->show();
}

void backendJobsQueue::hideWaitWidget()
{
    kDebug();
    if (!mWaitWidget) return;

    delete mWaitWidget;
    mWaitWidget = 0;
}

}

#include "backendjobsqueue.moc"

