/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef COMPOSER_EDITOR_H
#define COMPOSER_EDITOR_H

#include "ui_composereditorbase.h"

class KToolBar;
class KActionCollection;
class KAction;
class KActionMenu;
class KToggleAction;
class KHTMLPart;
class KUrl;
class KIconLoader;

namespace KBlogger
{

class KBloggerLinkDialog;
class Composer;
class kbloggerMedia;
class visualEditor;

class ComposerEditor : public QWidget, public Ui::composerEditorBase
{
    friend class Composer;
    Q_OBJECT

public:
    ComposerEditor( Composer *composer = 0, QWidget* parent = 0 );
    ~ComposerEditor();

    void insertMedia(kbloggerMedia* media);

public Q_SLOTS:
    void showUploadMediaDialog( const QString& filename = QString() );
    void showMediaList();

protected Q_SLOTS:
    void writeOnPreview(const QString& text);

private Q_SLOTS:
    //Visual Editor private slots.
    void toggleItalic();
    void toggleBold();
    void toggleUnderline();
    void toggleStrikeout();
    void toggleCode();
    void fontSizeMore();
    void fontSizeLess();

    void showLinkDialog();
    void addLink(const QString &, const QString& );
    void showHtmlDialog();


    //void addImageUploaded( const QString& url );

    void updateCharFmt();
    void spellchecking();

    void syncEditors(int i);
private:
    void createActions();
    QString htmlToRichtext(const QString& html); //TODO Move

    Composer *mComposer;
    visualEditor *visualTextEditor;
    KBloggerLinkDialog *mLinkDialog;

    QString mBlogname;

    KToolBar *mToolBar;
    KToolBar *mToolBarPreview;
    KToggleAction *boldAction;
    KToggleAction *italicAction;
    KToggleAction *underlineAction;
    KToggleAction *strikeoutAction;
    KAction *fontSizeMoreAction;
    KAction *fontSizeLessAction;
    KAction *codeAction;
    KAction *addLinkAction;
    KActionMenu *insertMediaMenu;
    KAction *newMediaAction;
    KAction *addMediaAction;
    KAction *addHtmlAction;
    KAction *spellCheckAction;

    KActionCollection *mActionCollection; //FIXME? Use application Action collection?

    KHTMLPart *PreviewBrowser;

    int prevPage;
};

} //namespace

#endif

