/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "itemsmanager.h"

#include <QUrl>
#include <QFile>

#include <kblog/blogmedia.h>
#include <kdebug.h>


#include "postslist.h"
#include "kbloggerpost.h"
#include "kbloggermedia.h"
#include "medialist.h"

#include "backend.h"
#include "mainwidget.h"
#include "importmediadialog.h"

namespace KBlogger
{
itemsManager *itemsManager::s_self = 0;

itemsManager *itemsManager::self(QObject* parent)
{
    if ( !s_self )
        s_self = new itemsManager(parent);
    return s_self;
}

itemsManager::itemsManager(QObject *parent): QObject(parent)
{
    kDebug();
}

itemsManager::~itemsManager()
{
    kDebug();
    s_self = 0;
}

void itemsManager::createNewPost(KBloggerPost *post)
{
    kDebug();

    //Insert the new post to upload in the widget, so the user can see this post.
    if (!localWidgetsList.contains(post->getBlogName())) {
        kError() << "\t draftWidgestList doesn't contain the blogName of the post" << endl;
        return;
    }
    localWidgetsList.value( post->getBlogName() )->appendPost(post);
}

void itemsManager::enqueuePostsToUpload( const QString& blogname )
{
    kDebug() << "itemsManager::enqueuePostsToUpload blogname="
    << blogname;

    //Uploads queue
    QList<KBloggerPost*> entriesUploadList;

    //getting the entries to upload.
    if ( blogname.isEmpty() ) {
        //Upload All posts
        QMapIterator<QString, localDraftPostsList*> i_draft(localWidgetsList);
        while (i_draft.hasNext()) {
            i_draft.next();
            //kDebug() << "\tSend:"<< i_draft.key() << ": " << i_draft.value() << endl;
            entriesUploadList += i_draft.value()->getEntriesList();
        }
    } else {
        //Upload all posts of specified blogname.
        entriesUploadList = localWidgetsList[blogname]->getEntriesList();

    }

    //TEST
    if ( !entriesUploadList.size() ) {
        kDebug() << "no post to upload";
    }

    backend *mBackend = backend::self();
    for (int i = 0; i < entriesUploadList.size(); ++i) {
        kDebug() << "\ti=" << i << endl;

        mBackend->sendPost( entriesUploadList.at(i) );
    }
}

void itemsManager::enqueueMediaToUpload( const QString& blogname )
{
    kDebug() << "itemsManager::enqueueMediaToUpload blogname="
    << blogname;;

    //Uploads queue
    QList<kbloggerMedia*> mediaUploadList;

    //Getting the media to upload.
    if ( blogname.isEmpty() ) {
        QMapIterator<QString, mediaList*> i_media(mediaWidgetsList);
        while (i_media.hasNext()) {
            i_media.next();
            mediaUploadList += i_media.value()->getMediaList();
        }
    } else {
        //Upload all media of specified blogname.
        mediaUploadList = mediaWidgetsList[blogname]->getMediaList();
    }

    //TEST
    if ( mediaUploadList.isEmpty() ) {
        kDebug() << "no media to upload";
    }

    //Creating jobs from media
    backend *mBackend = backend::self();
    for (int i = 0; i < mediaUploadList.size(); ++i) {
        kDebug() << "\ti=" << i << endl;

        if ( mediaUploadList.at(i)->status() == KBlog::BlogMedia::New )
            mBackend->sendMedia( mediaUploadList.at(i) );
    }
}

void itemsManager::addInTheWidgetsList( const QString& blogName, localDraftPostsList* widget )
{
    kDebug();
    localWidgetsList[blogName] = widget;
}

void itemsManager::addInTheWidgetsList(const QString& blogName, sentPostsList* widget)
{
    kDebug();
    sentWidgetsList[blogName] = widget;
}

void itemsManager::addInTheWidgetsList(const QString& blogName, trashedPostsList* widget)
{
    kDebug();
    trashedWidgetsList[blogName] = widget;
}

void itemsManager::addInTheWidgetsList( const QString& blogName, mediaList* widget)
{
    kDebug();
    mediaWidgetsList[blogName] = widget;
}

void itemsManager::removeDraftPost(KBloggerPost* kbPost)
{
    kDebug();

    if (!kbPost) {
        kError() << "\tcurrentPost is Zero, no problem!" << endl;
        return;
    }

    localWidgetsList.value( kbPost->getBlogName() )->removePost(kbPost, true);
}

void itemsManager::removeTrashedPost(KBloggerPost* kbPost)
{
    kDebug();

    if (!kbPost) {
        kError() << "\tcurrentPost is Zero, no problem!" << endl;
        return;
    }

    trashedWidgetsList.value( kbPost->getBlogName() )->removePost(kbPost, true);
}

void itemsManager::addToSentList(KBloggerPost* kbPost)
{
    kDebug();
    Q_ASSERT(kbPost);
    kDebug() << "\tadd entry in the " << kbPost->postId() << " sent widget" << endl;

    //Append post in the Sent Post List
    sentPostsList *spl = sentWidgetsList.value( kbPost->getBlogName() );
    Q_ASSERT(spl);
    spl->appendPost(kbPost);
}

void itemsManager::trashPost(KBloggerPost *kbPost)
{
    kDebug();
    if (!kbPost)
        return;
    trashedWidgetsList.value( kbPost->getBlogName() )->appendPost(kbPost);
}

void  itemsManager::moveDraftToSentList(KBloggerPost *kbPost)
{
    kDebug();
    if (!kbPost)
        return;

    localWidgetsList.value( kbPost->getBlogName() )->removePost(kbPost, false); addToSentList(kbPost);
}

void itemsManager::updatePostsSentList(const QString &blogname)
{
    kDebug();
    sentPostsList *mSentEntriesWidget;
    mSentEntriesWidget = sentWidgetsList[blogname];
    Q_ASSERT(mSentEntriesWidget);
    mSentEntriesWidget->updatePostsSentList();
}

bool itemsManager::isStored(KBloggerPost *kbPost)
{
    kDebug();
    if ( !kbPost ) return false;

    QList<KBloggerPost*> postsToScan;
    KBloggerPost *postToScan;

    postsToScan  = sentWidgetsList[ kbPost->getBlogName() ]->getEntriesList();
    postsToScan += localWidgetsList[ kbPost->getBlogName() ]->getEntriesList();
    postsToScan += trashedWidgetsList[ kbPost->getBlogName() ]->getEntriesList();

    for (int i = 0; i < postsToScan.size(); ++i) {
        kDebug() << "I=" << i;
        postToScan = postsToScan.at(i);

        if ( *kbPost == postToScan ) return true;
        if ( kbPost->postId() == postToScan->postId() ) {
            //Exists another post with the same ID. If it's in sent/trash it will deleted.
            postsList* containerWidget;
            containerWidget = qobject_cast<postsList*> ( postToScan->container() );
            if ( containerWidget ) {
                containerWidget->removePost( postToScan );
                return false;
            }
        }
    }
    return false;
}

void itemsManager::getUncachedMediaInThePosts( const QString &blogname )
{
    kDebug() << blogname;
    Q_ASSERT( !blogname.isEmpty() );
    QList<KBloggerPost*> postsList;
    postsList = sentWidgetsList.value( blogname )->getEntriesList();

    //Get Media Urls:
    QList<kbloggerMedia*> mediaList;
    mediaList = mediaWidgetsList.value( blogname )->getMediaList();

    QStringList uploadedMediaUrlList;
    foreach (kbloggerMedia* media, mediaList) {
        QString url;
        url = media->url().url();
        if (!url.isEmpty())
            uploadedMediaUrlList += url;
    }

    foreach (KBloggerPost* post, postsList) {
        /*
        QMap<const QString& uploadedMediaUrl, KBloggerPost*> mediaMap;
        mediaMap= post->getUploadedMedia();
        foreach( QString url, mediaMap){
          if ( !uploadedMediaUrlList.contains(url) )
          importMediaDialog()::self()->addUrl(url, mediaMap.value(url) );
        }
        */
        QStringList uploadedMediaUrlInThePost;
        uploadedMediaUrlInThePost = post->mediaUrlList();
        foreach( QString url, uploadedMediaUrlInThePost) {
            kDebug() << "cached Urls:" << uploadedMediaUrlInThePost << endl
            << "url in the post:" << url;
            if ( !uploadedMediaUrlList.contains(url) )
                importMediaDialog::self()->addUrl(url, post->getBlogName(), post->postId() );
        }
    }
    importMediaDialog::self()->exec();
}

void itemsManager::addMedia( kbloggerMedia *media )
{
    kDebug();
    if (!media) return;
    QString blogname = media->getBlogname();
    if (blogname.isEmpty()) return;
    mediaWidgetsList.value( blogname )->appendMedia(media);
}

QString itemsManager::url(QString blogname, quint16 checksum)
{
    kDebug();
    if (blogname.isEmpty()) {
        kError() << "blogname is empty";
        return QString();
    }
    if (!checksum) {
        kDebug() << "checksum is empty()";
        return QString();
    }
    QList<kbloggerMedia*> list;
    list = mediaWidgetsList.value( blogname )->getMediaList();
    for (int i = 0; i < list.size(); ++i) {
        if (list.at(i)->checksum() == checksum)
            return list.at(i)->url().url();
    }
    return QString();
}

QByteArray itemsManager::mediaData(const QUrl &url)
{
    kDebug() << url;

    //Get Blogname
    QString blogname = mainWidget::self()->currentBlogname();
    if (blogname.isEmpty()) return QByteArray();

    ///Search
    //Get Media List
    QList<kbloggerMedia*> mediaList;
    mediaList = mediaWidgetsList.value( blogname )->getMediaList();

    //Is it a local media?
    bool isLocalUrl = false;

    if ( url.toString().left(15) == "kblogger-media-" )
        isLocalUrl = true;

    foreach (kbloggerMedia* media, mediaList) {
        QString mediaUrl;

        if ( isLocalUrl )
            mediaUrl = media->cachedFilename();
        else
            mediaUrl = media->url().url();

        if ( !mediaUrl.isEmpty() && url == mediaUrl ) {
            kDebug() << "Media Data is OK";
            return media->data();
        }
    }

    //No media Found!
    return QByteArray();

}

const QList<kbloggerMedia*> itemsManager::getMediaList(const QString& blogname)
{
    kDebug();
    Q_ASSERT ( !blogname.isEmpty() );
    mediaList *ml = mediaWidgetsList.value( blogname );
    Q_ASSERT(ml);
    return ml->getMediaList();
}

} //namespace
#include "itemsmanager.moc"
