/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef ITEMSMANAGER_H
#define ITEMSMANAGER_H

#include <QtCore/QObject>

#include <QMap>

//#include <kdebug.h>
//#include <klocalizedstring.h>
//#include <kdatetime.h>

/**
 * @author Antonio Aloisio <gnuton@gnuton.org>
 */

//using namespace KBlog;

class QUrl;

namespace KBlogger
{

class  sentPostsList;
class  localDraftPostsList;
class  trashedPostsList;
class  mediaList;
class  kbloggerMedia;
class  KBloggerPost;

class itemsManager : public QObject
{
    Q_OBJECT

public:
    static itemsManager *self(QObject* parent = 0);
    ~itemsManager();

    //create a local draft that will be uploaded
    void createNewPost(KBloggerPost* post);

    //getPost read all the post that will be uploaded in every "Local Draft" widgets
    void enqueuePostsToUpload(const QString& blogname = QString());

    void addInTheWidgetsList( const QString& blogName, localDraftPostsList* widget );
    void addInTheWidgetsList( const QString& blogName, sentPostsList* widget);
    void addInTheWidgetsList( const QString& blogName, trashedPostsList* widget);
    void addInTheWidgetsList( const QString& blogName, mediaList* widget);

    //POSTS: Remove the post from the draft
    void removeDraftPost(KBloggerPost* kbPost);

    //POSTS: Remove the post from the trash
    void removeTrashedPost(KBloggerPost* kbPost);

    //POSTS: Add currentUploadItem in the sent QTreeWidget
    void addToSentList(KBloggerPost* kbPost);

    //POSTS: Add the kbPost in the trash QTreeWidget
    void trashPost(KBloggerPost *kbPost);

    //POSTS: Move
    void moveDraftToSentList(KBloggerPost *kbPost);

    //POSTS
    void updatePostsSentList(const QString &blogname);

    //POSTS Check the ID of fetched posts.
    bool isStored(KBloggerPost *kbPost);

    //POSTS Scans sent posts and put the uncached media ("http://www.yourblog.org/media.xxx") in the mediagetter dialog.
    void getUncachedMediaInThePosts( const QString &blogname );

    //MEDIA: Add the media in the correct QTreeWidget;
    void addMedia( kbloggerMedia *media );

    //MEDIA: Enqueue media in the upload List;
    void enqueueMediaToUpload(const QString& blogname = QString());

    //MEDIA: Returns the url of a media (identified with your data checksum)
    QString url( QString blogname, quint16 checksum );

    //MEDIA: Returns the data of the url stored in a kbloggerMedia Class.
    QByteArray mediaData(const QUrl &url);

    //MEDIA
    const QList<kbloggerMedia*> getMediaList(const QString& blogname);

protected:


private:
    //default costructor.
    itemsManager(QObject *parent = 0);
    static itemsManager *s_self;

    //Map <BlogName,WidgetThatContainsMediaOrPostsOfThisBlogName>
    QMap<QString, localDraftPostsList*> localWidgetsList;
    QMap<QString, sentPostsList*> sentWidgetsList;
    QMap<QString, trashedPostsList*> trashedWidgetsList;
    QMap<QString, mediaList*> mediaWidgetsList;
};

}

#endif
