/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/
#include "mainwidget.h"

#include <QStackedWidget>
#include <QTextBrowser>

#include <kstandarddirs.h>
#include <kiconloader.h>
#include <kactioncollection.h>
#include <kmessagebox.h>
//#include <konq_pixmapprovider.h> //libkonq

#include "profileconfig.h"
#include "backend.h"
#include "itemlist.h"
#include "postslist.h"
#include "medialist.h"
#include "kblogger.h"
#include "kbloggerpost.h"
#include "kbloggermedia.h"
#include "itemsmanager.h"
#include "kbloggerblog.h"
#include "visualeditor.h"

namespace KBlogger
{

mainWidget *mainWidget::s_self = 0;

mainWidget *mainWidget::self(KActionCollection *actionCollection, QWidget* parent )
{
    if ( !s_self ) {
        s_self = new mainWidget(actionCollection, parent);
        s_self->initialize();
    }
    return s_self;
}

mainWidget::mainWidget( KActionCollection *actionCollection, QWidget* parent ):
        QWidget(parent),
        mKblogger( qobject_cast<kblogger*> ( parent )),
        mActionCollection(actionCollection)
{
    kDebug();
    setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose); //Destroy on close
}

void mainWidget::initialize()
{
    kDebug();
    //Create StackedWidget
    stackedWidget = new QStackedWidget(postsListWidget);
    postsListWidget->layout()->addWidget(stackedWidget);

    //Create PreviewBrowser Widget
    previewBrowser = new visualEditor(previewWidget);
    previewBrowser->setReadOnly(true);
    previewWidget->layout()->addWidget(previewBrowser);



    populateAccountList();

    connect( blogListTree, SIGNAL( itemClicked( QTreeWidgetItem *, int) ),
             this, SLOT(changeShownEntriesList(QTreeWidgetItem *)));
    connect( stackedWidget, SIGNAL( currentChanged(int) ), SLOT(updateKActions(int)));

    readConfig();
}

mainWidget::~mainWidget()
{
    kDebug();
    //remove all windgets in stackedWidget;
    for (int i = stackedWidget->count(); i > 0; --i)
        delete stackedWidget->widget(i);
    delete stackedWidget;
    s_self = 0;

}

void mainWidget::viewEntry(KBloggerPost* post)
{
    kDebug();
    if (!post) return;
    QString text;
    text = "<div><H3><FONT COLOR=#a0b0ff>" + post->title() + "</font></H3></div>";
    text += "<div><i>&nbsp;" + post->categories().join(", ") + "</i></div>" ;
//  text+=i18n("<i>Creation Date<i>")+
    text += "<div>&nbsp;" + post->content() + "</div>";
    previewBrowser->setText(text);
}

void mainWidget::viewEntry(kbloggerMedia *media)
{
    kDebug();
    if (!media) return;
    QString text;
    text = "<div><b>Name:</b> " + media->name() + "</div>";
    text += "<div><b>Url:</b> " + media->url().url() + "</div>" ;
    text += "<div align='center'><img src='" + media->cachedFilename() + "' /></div>";

    previewBrowser->setText(text);
}

void mainWidget::openSelectedMedia()
{
    mediaList *mediaWidget;
    mediaWidget = qobject_cast<mediaList*>( stackedWidget->currentWidget() );
    if ( !mediaWidget) return;
    mediaWidget->openSelectedMedia();
}

KActionCollection* mainWidget::actionCollection() const
{
    return mKblogger->actionCollection();
}

void mainWidget::reloadSettings()
{
    kDebug();
    saveStatus();
    populateAccountList();
    backend::self()->populateBlogsList();
}

void mainWidget::saveStatus()
{
    kDebug();
    postsList *postsWidget;
    mediaList *mediaWidget;

    for (int i = stackedWidget->count() - 1; i > 0; --i) {
        if ( ( postsWidget = qobject_cast<postsList*> ( stackedWidget->widget(i) ) ) )
            postsWidget->save();
        else
            if ( (mediaWidget = qobject_cast<mediaList*> ( stackedWidget->widget(i) )) )
                mediaWidget->save();

    }
    saveConfig();
}

void mainWidget::updateAllSentPostList()
{
    kDebug();
    QString blogname = currentBlogname();
    if ( blogname.isEmpty() ) return;
    itemsManager::self()->updatePostsSentList( blogname );
}

void mainWidget::trashPost()
{
    kDebug();
    postsList *postsWidget;
    postsWidget = qobject_cast<postsList*>(stackedWidget->currentWidget() );
    if ( !postsWidget) {
        //KMessageBox::sorry( this, i18n("No Post selected."));
        return;
    }
    postsWidget->trashPost();

}

void mainWidget::deletePost()
{
    kDebug();
    postsList *postsWidget;
    postsWidget = qobject_cast<postsList*>(stackedWidget->currentWidget() );
    if ( !postsWidget) {
        //KMessageBox::sorry( this, i18n("No Post selected."));
        return;
    }
    postsWidget->deletePost();
}

QString mainWidget::currentBlogname()
{
    kDebug();
    itemList *itemWidget;
    itemWidget = qobject_cast<itemList*>( stackedWidget->currentWidget() );
    if ( itemWidget)
        return itemWidget->blogname();
    else
        return QString();
}

void mainWidget::modifyPost()
{
    kDebug();
    postsList *postsWidget;
    postsWidget = qobject_cast<postsList*>(stackedWidget->currentWidget() );
    if ( !postsWidget) {
        KMessageBox::sorry( this, i18n("No Post selected."));
        return;
    }
    postsWidget->modifyPost();
}

void mainWidget::removeLocalMedia()
{
    kDebug();
    mediaList *mediaWidget;
    mediaWidget = qobject_cast<mediaList*>( stackedWidget->currentWidget() );
    if ( !mediaWidget) {
        KMessageBox::sorry( this, i18n("No Media selected."));
        return;
    }
    mediaWidget->removeLocalItem();
}

void mainWidget::uploadCurrentMedia()
{
    kDebug();
    QString blogname = currentBlogname();
    if ( blogname.isEmpty() ) return;
    itemsManager::self()->enqueueMediaToUpload( blogname );
}

void mainWidget::uploadCurrentPosts()
{
    kDebug();
    QString blogname = currentBlogname();
    if ( blogname.isEmpty() ) return;
    itemsManager::self()->enqueuePostsToUpload( blogname );
}

void mainWidget::populateAccountList()
{
    kDebug();

    //A.Declare
    QTreeWidgetItem *root;
    QTreeWidgetItem *item;
    int index = 0;

    //B. Clean Map,QWidgetTree and QStackWidget
    mQTreeWidgetItemMap.clear(); //FIXME MEMORY LEAK IN TREEINDEX!!
    blogListTree->clear();
    //prevent a memory lack!
    for (int i = stackedWidget->count(); i >= 0; --i)
        delete stackedWidget->widget(i);

    //C. Add Welcome Screen
    {
        QTextBrowser* welcomeBrowser;
        welcomeBrowser = new QTextBrowser(this);
        stackedWidget->addWidget(welcomeBrowser);
        root = new QTreeWidgetItem(blogListTree);
        root->setText( 0, i18n("Blogs") );
        root->setIcon( 0, KIcon("kblogger"));
        root->setExpanded (true);
        TreeIndex welcomeIndex(index, static_cast<QWidget*> (welcomeBrowser));
        mQTreeWidgetItemMap.insert( root, welcomeIndex );
        populateWelcomeTextBrowser(welcomeBrowser);
        blogListTree->insertTopLevelItem( 0, root );
    }

    //D. Populate
    {
        QList<kbloggerBlog> blogList;
        blogList = backend::self()->getBlogList();
        for ( int i = 0; i < blogList.size(); ++i) {
            kbloggerBlog blog;
            QString accountName;
            QStringList childrenList;
            QList<QTreeWidgetItem *> children;
            QList<KIcon> iconList;
            QList<QWidget*> widgetList;

            blog = blogList.at(i);

            //Add Parent Item
            item = new QTreeWidgetItem(root);
            item->setText( 0, blog.blogname() );
            item->setIcon( 0, blog.icon() );
            //blogListTree->insertTopLevelItem( 0, item );

            //Create & Add postsList in the StackWidget.
            accountName = item->text(0);
            sentPostsList *mSentEntries;
            localDraftPostsList *mLocalEntries;
            trashedPostsList *mTrashEntries;
            mediaList *mMedia;

            mSentEntries = new sentPostsList(accountName, mKblogger, this);
            mLocalEntries = new localDraftPostsList(accountName, mKblogger, this);
            mTrashEntries = new trashedPostsList(accountName, mKblogger, this);
            mMedia = new mediaList(accountName, mKblogger, this);

            TreeIndex parendIndex(++index,
                                  static_cast<QWidget*> (mSentEntries),
                                  accountName
                                 );
            mQTreeWidgetItemMap.insert( item, parendIndex );

            stackedWidget->addWidget(mSentEntries);
            stackedWidget->addWidget(mLocalEntries);
            stackedWidget->addWidget(mMedia);
            stackedWidget->addWidget(mTrashEntries);

            //Add Children Items
            childrenList << i18n( "Local entries" )
            << i18n( "Media" )
            << i18n( "Trash" );

            iconList << KIcon( "mail-message-new" )
            << KIcon("fileview-preview")
            << KIcon("user-trash");

            widgetList << static_cast<QWidget*> ( mLocalEntries )
            << static_cast<QWidget*> ( mMedia )
            << static_cast<QWidget*> ( mTrashEntries );

            for (int i = 0; i < childrenList.size() ; ++i) {
                QTreeWidgetItem* childrenItem;
                childrenItem = new QTreeWidgetItem( (QTreeWidget*)0,
                                                    QStringList(childrenList.at(i)));
                TreeIndex childIndex(++index,
                                     static_cast<QWidget*> (widgetList.at(i)),
                                     childrenList.at(i)
                                    );
                mQTreeWidgetItemMap.insert(childrenItem, childIndex);
                children.append(childrenItem);
                children.at(i)->setIcon(0, iconList.at(i));
            }
            item->addChildren(children);


        }
    }
    updateItemNumbers();
}

void mainWidget::changeShownEntriesList(QTreeWidgetItem *currentItem)
{
    kDebug();
    TreeIndex treeIndexFromMap( mQTreeWidgetItemMap.value(currentItem) );
    int row = treeIndexFromMap.pos();
    kDebug() << "mainWidget::changeShownEntriesList() selected row:" << row << endl;
    stackedWidget->setCurrentIndex(row);
}

void mainWidget::updateKActions(int i)
{
    kDebug();

    QString enabledActionGroup;

    switch (i) {
    case - 1: {
        mKblogger->updateSearchLine();
        return;
    }
    break;

    case 0: { //Welcome Screen
        mKblogger->updateSearchLine();
    }
    break;

    default: {
        //Sent, Trash & Modify entries Lists
        postsList *entriesWidget;
        entriesWidget = qobject_cast<postsList*>( stackedWidget->widget(i) );
        if ( entriesWidget ) {
            enabledActionGroup = "postActionGroup";
            entriesWidget->updateSearchLine();
        }
        //Media List
        mediaList *mediaWidget;
        mediaWidget = qobject_cast<mediaList*>( stackedWidget->widget(i) );
        if ( mediaWidget ) {
            enabledActionGroup = "mediaActionGroup";
            mediaWidget->updateSearchLine();
        }
    }
    }
    QList<QActionGroup *> actionGroupList;
    actionGroupList = mKblogger->actionCollection()->actionGroups();

    for (int x = 0; x < actionGroupList.size(); ++x) {
        bool enable;

        QActionGroup *actionGroup;
        actionGroup = actionGroupList.at(x);

        if ( !actionGroup ) continue;

        if ( i == 0 ) //Welcome Screen - All KActions are disabled.
            enable = false;
        else if ( actionGroup->objectName() == enabledActionGroup )
            enable = true;
        else if ( actionGroup->objectName() == "kblogActionGroup" )
            enable = true;
        else
            enable = false;

        kDebug() << "grupName=" << actionGroup->objectName() << "Enabled=" << enable;

        actionGroup->setEnabled(enable);
    }
}

void mainWidget::populateWelcomeTextBrowser(QTextBrowser* textBrowser)
{
    kDebug();

    QString html = "<div align=center><img src='";
    html += KStandardDirs::locate("appdata", "pics/kbloggerWelcome.jpg");
    html += "' /></div>";

    textBrowser->setHtml(html);
}

void mainWidget::saveConfig()
{
    kDebug();
    KSharedConfigPtr config = KSharedConfig::openConfig();
    KConfigGroup qsplitterGroup( config, "MainWindow QSplittersSizes" );
    qsplitterGroup.writeEntry("mainHSplitterSizes", mainHSplitter->sizes() );
    qsplitterGroup.writeEntry("mainVSplitterSizes", mainVSplitter->sizes() );
    qsplitterGroup.config()->sync();

}

void mainWidget::readConfig()
{
    kDebug();
    KSharedConfigPtr config = KSharedConfig::openConfig();
    QList<int> splitterHsizes, splitterVsizes;
    KConfigGroup qsplitterGroup( config, "MainWindow QSplittersSizes" );
    splitterHsizes = qsplitterGroup.readEntry("mainHSplitterSizes", splitterHsizes );
    splitterVsizes = qsplitterGroup.readEntry("mainVSplitterSizes", splitterVsizes );

    if ( !splitterHsizes.isEmpty() )
        mainHSplitter->setSizes( splitterHsizes );
    if ( !splitterVsizes.isEmpty() )
        mainVSplitter->setSizes( splitterVsizes );
}

void mainWidget::updateItemNumbers()
{
    kDebug();

    QMapIterator<QTreeWidgetItem *, TreeIndex> i(mQTreeWidgetItemMap);
    while (i.hasNext()) {
        i.next();

        postsList *entriesListWidget = 0;
        mediaList *mediaListWidget = 0;

        QTreeWidgetItem *item = i.key();
        TreeIndex treeIndexFromMap( i.value() );
        int count = -1;

        entriesListWidget = qobject_cast<postsList*>( treeIndexFromMap.widget() );
        if (entriesListWidget) {
            count = entriesListWidget->count();
        } else {
            mediaListWidget = qobject_cast<mediaList*>( treeIndexFromMap.widget() );
            if (mediaListWidget) {
                count = mediaListWidget->count();
            }
        }
        if (count < 0) continue;
        item->setText(0, QString("%1 (%2)").arg(treeIndexFromMap.text()).arg(count));
    }
}

} //namespace

#include "mainwidget.moc"
