/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KBLOGGERMAINWIDGET_H
#define KBLOGGERMAINWIDGET_H

#include <qmap.h>
#include "ui_mainwidgetbase.h"
#include "postslist.h"

//class profileConfig;
class QStackedWidget;
class QTextBrowser;
class KIconLoader;
class KActionCollection;

namespace KBlogger
{

class backend;
class KBloggerPost;
class kbloggerMedia;
class kblogger;
class visualEditor;

class TreeIndex
{
public:
    TreeIndex(int pos, QWidget* widget, const QString& text = QString() ) {
        mPos = pos; mWidget = widget; mText = text;
    };
    TreeIndex() {};
    ~TreeIndex() {};

    int pos() {
        return mPos;
    }
    QWidget* widget() {
        return mWidget;
    }
    QString text() {
        return mText;
    }
private:
    int mPos; //This is the position in the QTreeWidget.
    QWidget* mWidget; //this is the widget that QStack should show when this item is clicked.
    QString mText; //This is the original text of this QTreeWidgetItem;
};

class mainWidget : public QWidget, public Ui::mainWidgetBase
{
    Q_OBJECT
    friend class postsList;
    friend class mediaList;

public:
    static mainWidget *self(KActionCollection *actionCollection = 0, QWidget* parent = 0);

    virtual ~mainWidget();
    void viewEntry(KBloggerPost* post);
    void viewEntry(kbloggerMedia *media);
    void openSelectedMedia();
    KActionCollection* actionCollection() const;

public Q_SLOTS:
    void reloadSettings();
    void saveStatus();

    /*KActions slots*/
    void updateAllSentPostList();
    void trashPost();
    void deletePost();
    void modifyPost();
    void removeLocalMedia();

    void uploadCurrentMedia();
    void uploadCurrentPosts();

    QString currentBlogname(); //Return the blogname of the shown widget

protected:
    void updateItemNumbers();

private Q_SLOTS:
    //read the config and fill the AccountList.
    void populateAccountList();
    //When you click a QTreeWidgetItem in the blog list this func is called.
    void changeShownEntriesList(QTreeWidgetItem *currentItem);
    void updateKActions(int i);

private:
    mainWidget( KActionCollection *actionCollection, QWidget* parent = 0 );
    void initialize();
    void populateWelcomeTextBrowser(QTextBrowser* textBrowser);
    void saveConfig();
    void readConfig();

private:
    static mainWidget *s_self;
    QStackedWidget *stackedWidget;
    QMap<QTreeWidgetItem *, TreeIndex > mQTreeWidgetItemMap;
    kblogger* mKblogger;
    KActionCollection *mActionCollection;
    visualEditor *previewBrowser;

Q_SIGNALS:
    void activated(int); //sent by "Draft" or "Sent" in the blogList
};

}//namespace
#endif
