/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef MEDIALIST_H
#define MEDIALIST_H

#include "itemlist.h"
#include "ui_medialistbase.h"

#include <QMap>

#include "backend.h"

class QTreeWidgetItem;

class KBlog::BlogMedia;
namespace KBlogger
{

class KBloggerPost;
class kbloggerMedia;
class kblogger;

class mediaListWidget : public itemList, public Ui::mediaListBase
{
    Q_OBJECT

public:
    mediaListWidget( const QString& blogname,
                     QWidget* parent = 0 );

    virtual ~mediaListWidget();

    //Appends media in the widget, map and list
    virtual void appendMedia( kbloggerMedia* kbMedia );

    //Returns a ptr to the current selected media in the widget
    kbloggerMedia* getCurrentMedia();

    //Returns the number of item stored in the widget (map)
    int count();

protected:
    //Widget Coloumns
    enum coloumns { Name = 0, CreationDate, Size, Status };

    //Store kbloggerMedia ptr sorted by its QTreeWidgetItem ptr.
    QMap<QTreeWidgetItem*, kbloggerMedia*> mMediaMap;

    //Store kbloggerMedia ptr sorted by creation
    QList<kbloggerMedia*> mSortedMediaList;
};


/**
 * mediaList Class
 */
class mediaList : public mediaListWidget
{
    Q_OBJECT
public:
    mediaList( const QString& blogname,
               kblogger* kbloggerParent,
               QWidget* parent = 0 );

    virtual ~mediaList();

    //
    void appendMedia( kbloggerMedia* kbMedia );

    //Save All media (QList sorted) in a file
    void save();

    //Reads Data from file and creates media
    void load();

    //Update the search Line in the KBlogger class
    void updateSearchLine();

    //Returns a list of media stored in this widget
    QList<kbloggerMedia*> getMediaList();

    //Remove  Media (Polish internal vars and delete local cached copy of the media)
    void removeLocalItem();

public Q_SLOTS:
    //Runs Application associated to the mime of the media.
    void openSelectedMedia();

private Q_SLOTS:
    //Shows a preview of the media
    void previewItem();

    //Show a popup over the QTreeWidgetItem of the media
    void showPopup(const QPoint& pos);

private:
    kblogger *mKblogger; //FIXME Use kblogger::self()
    //Filename stores the media informations
    //it will be saved and loaded by save() and load() functions
    QString mFilename;

};

}
#endif

