/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KBLOGGERENTRIESLIST_H
#define KBLOGGERENTRIESLIST_H

#include <qlist.h>

#include "itemlist.h"
#include "ui_postslistbase.h"
//#include "backendmanager.h" // KBloggerPost

/**
 * Generic Posts List widget.
 * Please don't use this class.
 * Use sentPostsList or localDraftPostsList instead.
 */

class KIconLoader;

namespace KBlogger
{

class backend;
class mainWidget;
class kblogger;
class KBloggerPost;

class postsList : public itemList, public Ui::postsListBase
{
    Q_OBJECT
public:
    enum entriesType { sent, local, trashed };
    enum coloumns { blog = 0, id, title, categories, date, content, published };

    //Default Constructor
    postsList( const QString& blogName,
               entriesType type,
               kblogger* kbloggerParent,
               QWidget* parent = 0 ); //List entries in upload

    //Default Destructor
    virtual ~postsList();

    // Remove an entry from the widget
    void removePost( KBloggerPost* postToDelete,  bool delPost = false);

    // store mPostList in a file
    void save(); //save mPostList in a file

    //restore mPostList from a file
    void load();

    //get mPostList
    QList<KBloggerPost*> getEntriesList();

    //Append a entry in the widget
    void appendPost( KBloggerPost* post);

    //@return the number of entries stored in mPostList
    int count();

    //Performs some delete actions on the selected item
    void trashPost();

    //Delete locally a post
    void deletePost();

    // Update search line of kblogger class.
    void updateSearchLine();

    //returns the type of widget that contains this post.
    //entriesType whereIsThisPost();

public Q_SLOTS:
    //
    void modifyPost();

protected:
    void setFilename(const QString& account, entriesType type); //sets mFilename variable.
    backend *mBackend;

private Q_SLOTS:
    //show a preview of mPost
    void previewItem();
    void showPopup(const QPoint& pos);

private:
    KBloggerPost* getCurrentPost();

private:
    mainWidget *mMainWidget;

    kblogger *mKblogger;
    entriesType mType;
    QMap<QTreeWidgetItem*, KBloggerPost*> mPostsMap;
    QString mFilename;

};


/**
 *  This widget shows the list of sent entries
 */

class sentPostsList : public postsList
{
    Q_OBJECT
public:
    sentPostsList( const QString& blogName,
                   kblogger* kbloggerParent,
                   QWidget* parent = 0 ); //List fetched entries

    virtual ~sentPostsList();

    void updatePostsSentList();

};


/**
 * This Widget shows the draft entries (to upload or not)
 */
class localDraftPostsList : public postsList
{
    Q_OBJECT

public:
    localDraftPostsList( const QString& blogName,
                         kblogger* kbloggerParent,
                         QWidget* parent = 0 ); //List entries in upload
    virtual ~localDraftPostsList();
};

/**
 * This Widget shows the draft entries (to upload or not)
 */
class trashedPostsList : public postsList
{
    Q_OBJECT

public:
    trashedPostsList( const QString& blogName,
                      kblogger* kbloggerParent,
                      QWidget* parent = 0 ); //List entries in upload
    virtual ~trashedPostsList();
};
} //namespace

#endif
