/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "profilelist.h"

#include <kinputdialog.h>
#include <kconfigdialog.h>

#include "profileconfig.h"
#include "profileconfigdialog.h"

namespace KBlogger
{

profileList::profileList( QWidget* parent ): QWidget(parent)
{
    kDebug();
    setupUi(this);
    populateList();

    connect(addButton, SIGNAL(clicked()), SLOT(addAccount()));
    connect(modifyButton, SIGNAL(clicked()), SLOT(modifyAccount()));
    connect(delButton, SIGNAL(clicked()), SLOT(delAccount()));
}

profileList::~profileList()
{
    kDebug();
}

void profileList::populateList()
{
    //profileListTree
    QStringList groupList;
    QList<QTreeWidgetItem *> items;

    //TODO Move this code -begin-
    profileConfig *mProfileconfig = profileConfig::prefs("THIS_ACCOUNT_DOESNT_EXIST");
    groupList = mProfileconfig->config()->groupList();
    kDebug () << "profileConfigDialog::populateAccountComboBox()" <<  groupList << endl;
    groupList = groupList.filter( QRegExp("^Account_"));
    groupList.replaceInStrings(QRegExp("^Account_"), QString());
    //TODO Move this code -end-

    profileListTree->clear();

    for (int i = 0; i < groupList.size() ; ++i)
        items.append(new QTreeWidgetItem((QTreeWidget*)0,
                                         QStringList(groupList.at(i))));
    profileListTree->insertTopLevelItems(0, items);

}

void profileList::addAccount()
{
    QStringList groupList;
    QString newAccount;
    QRegExp testRegExp("[.a-z0-9A-Z]+");
    QValidator* validator = new QRegExpValidator(testRegExp, this);

    newAccount = KInputDialog::getText(
                     i18n("Add a new profile"),
                     i18n("Insert here your new account name:"),
                     QString(), 0, this, validator);
    kDebug() << "profileList::addAccount" << newAccount << endl;

    //TODO Move this code -begin-
    profileConfig *mProfileconfig = profileConfig::prefs("THIS_ACCOUNT_DOESNT_EXIST");
    groupList = mProfileconfig->config()->groupList();
    //TODO Move this code -end-

    //test TODO
    if (newAccount.isEmpty()) return;

    //create account
    mProfileconfig = profileConfig::prefs(newAccount.prepend("Account_"));
    mProfileconfig->setUrl("http://www.yourblog.org/xmlrpc.php");
    mProfileconfig->writeConfig();
    populateList();
}

void profileList::delAccount()
{
    QString account;
    QTreeWidgetItem *mCurrentTreeItem;
    profileConfig *mProfileconfig;
    mCurrentTreeItem = profileListTree->currentItem();
    if (!mCurrentTreeItem) return;
    account = mCurrentTreeItem->text(0);
    account = account.prepend("Account_");
    kDebug() << "profileList::delAccount " << account << endl;
    mProfileconfig = profileConfig::prefs(account);
    mProfileconfig->setDefaults();
    mProfileconfig->deletePrefs(account);
    mProfileconfig->writeConfig();
    populateList();
}

void profileList::modifyAccount()
{
    QString account;
    KConfigDialog *dialog;
    profileConfigDialog *mProfileconfigDialog;
    profileConfig *mProfileconfig;
    QTreeWidgetItem *mCurrentTreeItem;

    mCurrentTreeItem = profileListTree->currentItem();
    if (!mCurrentTreeItem) return;
    account = mCurrentTreeItem->text(0);
    account = account.prepend("Account_");
    kDebug() << "profileList::modifyAccount " << account << endl;

    mProfileconfig = profileConfig::prefs(account);

    if ( KConfigDialog::showDialog( "profiles" ) )  {
        return;
    }
    mProfileconfigDialog = new profileConfigDialog(0);
    dialog = new KConfigDialog(this, "profiles", mProfileconfig);
    dialog->setFaceType(KPageDialog::Plain);
    //mProfileconfigWidget=(QWidget*) mProfileconfigDialog;
    //dialog->addPage(mProfileconfigWidget, i18n("Account profiles"), "users");
    dialog->addPage(mProfileconfigDialog, account.replace("_", ": ") , "users");

    dialog->setAttribute( Qt::WA_DeleteOnClose );
    dialog->show();
}

void profileList::insertItem()
{
    kDebug();
}

} //namespace kblogger
#include "profilelist.moc"
