/***************************************************************************

   Copyright (C) 2005-2007 by Christian Weilbach <christian_weilbach@web.de>
   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "uploadmediadialog.h"

#include <QFileInfo>
#include <QLabel>
#include <QLayout>


#include <kfiledialog.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kdebug.h>
#include <kurlrequester.h>
#include <kvbox.h>

#include "kbloggerconfig.h"
#include "imagemanager.h"
#include "backend.h"
#include "kbloggermedia.h"
#include "blogchooser.h"
#include "itemsmanager.h"
#include "composereditor.h"

const int MAX_FILE_SIZE_WARNING = 1048576; // 1Mb

namespace KBlogger
{

uploadMediaDialog::uploadMediaDialog( const QString& filename,
                                      const QString& blogname,
                                      QWidget *parent)
        : QDialog(parent),
        mMedia(0),
        mBlogname(blogname),
        fileJob(0),
        mBlogChooser(0),
        mImageManager(0)
{
    kDebug();
    setupUi(this);
    setAttribute( Qt::WA_DeleteOnClose ); //Destroy on close

    //FIXME This line permit to QTextEdit to display media, exist another solution?
    //QDir::setCurrent ( QDir::homePath() );

    //Add Blog Chooser
    if (blogname.isEmpty()) {
        mBlogChooser = new blogChooser(blogGroupBox);
        blogGroupBox->layout()->addWidget(mBlogChooser);
    } else {
        QLabel *blognameLabel = new QLabel(mBlogname, blogGroupBox);
        blogGroupBox->layout()->addWidget(blognameLabel);
    }


    /*
        //Filter
        switch (filter) {
        case Images: {
            genericGroupBox->setHidden(false);
            FilekURLRequester->setFilter( "Images ( *.png *.jpg *.gif )" );
            //mImageManager = new KBloggerImageManager(cacheDir, genericGroupBox, "Image Manager");
            mImageManager->show();
            layout()->addWidget( mImageManager);//, 0, 0 );
            connect(mImageManager, SIGNAL(imageResized(const QString&)), SLOT(setServerFileName(const QString&)));
            adjustSize();
        }
        break;
     */

    //CONNECT
    connect ( FilekURLRequester, SIGNAL(  textChanged(const QString&)),
              this, SLOT( setServerFileName(const QString&)) );
    connect ( enqueueButton, SIGNAL ( clicked() ),
              this, SLOT( enqueueSlot() ));
    //TEST
    if ( !filename.isEmpty() ) {
        FilekURLRequester->setUrl( filename );
    }
    exec();
}

uploadMediaDialog::~uploadMediaDialog()
{
    kDebug() << kbloggerMedia::cachePath();
    //FIXME This line permit to QTextEdit to display media, exist another solution?
    //QDir::setCurrent ( kbloggerMedia::cachePath() );

    if ( mBlogChooser ) delete mBlogChooser;
    if ( fileJob ) delete fileJob;

}

void uploadMediaDialog::enqueueSlot()
{
    kDebug();
    if ( mBlogChooser ) {
        if ( mBlogChooser->blogCombo->currentIndex() == 0 ) {
            return;
        }
    }


    if ( FilekURLRequester->url().isEmpty() ) {
        KMessageBox::error( 0, i18n( "You have to enter a name and a valid URL." )  );
        return;
    }

    //FIXME UploadDialog hungs up if the choose file is not readable..

    if (!fileJob) delete fileJob;
    fileJob = KIO::get( FilekURLRequester->url() );

    connect( fileJob, SIGNAL( result( KJob* ) ),
             this, SLOT( handleFileJobResult( KJob* ) ) );
    connect( fileJob, SIGNAL( data( KIO::Job*,  const QByteArray & ) ),
             this, SLOT( handleFileJobData( KIO::Job*,  const QByteArray & ) ) );
    connect( fileJob, SIGNAL( mimetype( KIO::Job*,  const QString & ) ),
             this, SLOT( handleFileJobMimetype( KIO::Job*,  const QString & ) ) );

    enqueueButton->setEnabled(false);
}

void uploadMediaDialog::handleFileJobData( KIO::Job* job, const QByteArray& data )
{
    Q_UNUSED(job);
    fileJob->disconnect( SIGNAL( data( KIO::Job*,  const QByteArray & ) ) );
    kDebug();// << "data:" << data << endl;
    if ( data.isNull() ) {
        kError() << "data is NULL" << endl;
        mediaData = 0;
        return;
    }
    if ( data.isEmpty() ) {
        kError() << "data is EMPTY" << endl;
        mediaData = 0;
        return;
    }
    mediaData = data;

}

void uploadMediaDialog::handleFileJobMimetype( KIO::Job* job, const QString& mimetype )
{
    Q_UNUSED(job);
    kDebug();
    fileJob->disconnect( SIGNAL( mimetype( KIO::Job*,  const QString & ) ) );
    mediaMimetype = mimetype;
}

void uploadMediaDialog::handleFileJobResult( KJob* job )
{
    kDebug();
    if ( job->error() ) {
        //job->showErrorDialog();
        KMessageBox::sorry ( this, i18n ( "This file is readable, isn't it?"));
        close();
        return;
    }
    if (mBlogname.isEmpty())
        mBlogname = mBlogChooser->blogCombo->currentText();
    kDebug() << "media created with success" << endl;

    mMedia = new kbloggerMedia( mediaData,
                                NameLineEdit->text(),
                                mediaMimetype,
                                mBlogname
                              );

    if ( mMedia->isCached() ) {
        itemsManager::self()->addMedia(mMedia);
        insertMediaInTheEditor();
    } else {
        QString msg = i18n("Impossible copy the file %1 in the cache",
                           FilekURLRequester->url().url() );

        KMessageBox::sorry ( this, msg );
    }

    close();
}

void uploadMediaDialog::setServerFileName(const QString& filename)
{
    kDebug();

    double filesize;

    if (filename.isEmpty()) return;

    FilekURLRequester->setUrl(filename);
    QFileInfo fileToUpload(filename);
    NameLineEdit->setText(fileToUpload.fileName());
    filesize = fileToUpload.size();
    if ( !filesize ) {
        filesizeLabel->setText(i18n("<font color='red'>This file is not valid</font>"));
    } else {
        filesize = filesize / 1024; //Convert byte in KByte
        if ( filesize > MAX_FILE_SIZE_WARNING ) {
            filesizeLabel->setText(i18n ("File size: <font color='red'>") + QString::number(filesize) + i18n("</font> Kb"));
            KMessageBox::sorry ( this, i18n ( "You can't upload big files!!" ) );
        } else {
            filesizeLabel->setText(i18n ("File size: <font color='green'>") + QString::number(filesize) + i18n("</font> Kb"));
            enqueueButton->setEnabled(true);
        }
        if (mImageManager)
            mImageManager->setImage(filename);
    }
}

void uploadMediaDialog::insertMediaInTheEditor()
{
    kDebug();
    ComposerEditor *c = qobject_cast<ComposerEditor*> (parent());
    if (!c) return;
    c->insertMedia(mMedia);
}

} //namespace

#include "uploadmediadialog.moc"
