/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "visualeditor.h"

#include <QList>
#include <QUrl>
#include <QFile>

#include <kdebug.h>

#include "itemsmanager.h"

namespace KBlogger
{
visualEditor::visualEditor(QWidget *parent): KTextEdit(parent)
{
    kDebug();
    setAcceptDrops(true);
}

visualEditor::~visualEditor()
{
    kDebug();
}

QVariant visualEditor::loadResource ( int type, const QUrl & name )
{
    kDebug() << type << name;
    //TODO Something
    if ( type != QTextDocument::ImageResource) return QVariant();

    QByteArray data;
    data = itemsManager::self()->mediaData(name); //TODO
    if ( !data.count() ) {
        kError() << "Data is NULL";
        return QVariant();
    }
    QVariant dataV(data);
    return dataV;

    /*
    QByteArray data;
    QFile file("/home/kdehack/browser/images/dmx.jpg");
    Q_ASSERT( file.open(QIODevice::ReadOnly));

    data = file.readAll();

    Q_ASSERT( data.count() );
    QVariant dataV(data);
    return dataV;*/

}

void visualEditor::dragEnterEvent(QDragEnterEvent *event)
{
    kDebug();
    //if (event->mimeData()->hasFormat("text/plain"))
    const QMimeData *mimeData = event->mimeData();
    if (mimeData->hasColor()) return;
    if (mimeData->hasImage ()) return;
    event->acceptProposedAction();
}

void visualEditor::dropEvent(QDropEvent *event)
{
    kDebug();
    const QMimeData *mimeData = event->mimeData();

    if ( mimeData->hasUrls() ) {
        QList<QUrl> urls;
        urls = mimeData->urls();
        for (int i = 0; i < urls.count() && i < 32; ++i) {
            QString url = urls.at(i).path();
            if (!url.isEmpty()) {
                emit droppedMedia(url);
            }
        }
    } else
        if ( mimeData->hasText() ) {
            insertHtml( mimeData->html() );
        } else
            if (mimeData->hasText()) {
                insertPlainText(mimeData->text());
            } else {
                return;
            }
    //ComposerTabWidget->setCurrentIndex(0);
    event->acceptProposedAction();
}

}

#include "visualeditor.moc"

